=begin word frequency.rb
AE 2020-01-18---Sat---05:05 - program outline
AE 2020-01-18---Sat---05:21 - completed program outline
AE 2020-01-18---Sat---11:05 - began writing program
AE 2020-01-19---Sun---15:30 - finished writing program
AE 2020-01-20---Mon---04:32 - another 90 minutes writing the frequency table application

This program has 18 words/concepts/functions as part of its application.

create datastructure:
    array frequency

read file
    open filename        

read line
get line_signal (first character of line)
if line_signal != number discard line
else read and discard words until , then read_and_count words to end of line

read_and_count
    read word (space separated)
        check if word in frequency
        if yes then increment value for that word
        if no then append word to frequency and set value = 1
        
summarize:
    print length of frequency
    sort frequency by value descending
    print records by value
    
done

Using Globals, Ruby can be written in a Forth programming style, defining application specific words and essentially simple single line definitions that are easy to understand, and then building the application from the use of those simple words.
=end

def setupApp
# file handling
    infile="uruk_iv.txt"
    $cdli = File.open(infile)  #file handle
    outfile="uruk_iv_signs.txt"
    $fout = File.open(outfile, "w")
    freqfile = "uruk_iv_freq.txt"
    $freqout = File.open(freqfile, "w")

# data structures
    $simples = []  # array of occurrences
    $compounds = []
    $discards = []
    $signfreq = {}  # hash holding the frequency table
    $signpop = {} # has holding the frequency table sorted by popularity (largest first)
end


def next_line
   $cdli.gets
end


def show_signs
   print "Simple Signs ("; print($simples.uniq.length); print") :"; puts $simples.uniq.sort.join("   ")
   puts
   print "Compound Signs ("; print($compounds.uniq.length); print") :"; puts $compounds.uniq.sort.join("   ")
   puts
   print "Discards ("; print($discards.uniq.length); print") :"; puts $discards.uniq.sort.join("   ")
end

def fout_show_signs
   $fout.write "Simple Signs ("; $fout.write($simples.uniq.length); $fout.puts") :"; 
        $fout.puts $simples.uniq.sort.join("\n"); $fout.write("\n\n")
   $fout.write "Compound Signs ("; $fout.write($compounds.uniq.length); $fout.puts") :"; 
        $fout.puts $compounds.uniq.sort.join("\n"); $fout.write("\n\n")
   $fout.write "Discards ("; $fout.write($discards.uniq.length); $fout.puts") :"; 
        $fout.puts $discards.uniq.sort.join("\n"); $fout.write("\n\n")
end

def clear_signs
   $simples=[]
   $compounds=[]
   $discards=[]
end

# extract using regexes
def betw_bracks(word)
    word.sub(/\[*([^\]]*)\]*/,'\1')  # extracts between [ ] and replaces with first grouping
end

def betw_parens(word)
    word.sub(/\(*([^\]]*)\)*/,'\1')  # extracts between ( ) and replaces with first grouping
end


# CDLI transliteration
   
def is_num?(char)  
  if char<"0" or char>"9" then false else true end
end

def is_compound?(word)
  if word[0]=="|" then true else false end
end

def translit_line?(line)
   # CDLI transliteration lines always have line numbers.  No other line does.
   is_num?(line[0])
end

def next_translit
   until translit_line?(line=next_line) do end
   line
end

def sift_words(line)
   line.split.each { |word| 
     if is_compound?(word) then $compounds.push(word)
     else 
       if is_num?(word[0]) then $discards.push(word)
       elsif word.match(/^,$/) then $discards.push(word)  # discards ,
       elsif word.match(/^\[\.+\]$/) then $discards.push(word)  # discards [...]
       else 
          word_extract = betw_parens(betw_bracks(word))
          if is_num?(word_extract[0]) then $discards.push(word_extract)
          elsif word_extract.match(/\.{3,}/) then $discards.push(word_extract)  # discards ...
          else 
            $simples.push(word_extract.split(/#/)[0].split(/@/)[0].split(/\?/)[0].split(/~/)[0])   # cleanup #, @, ?, ~
          end
       end
     end
   }
end

def all_words
   $cdli.each_line { |line| if translit_line?(line) then sift_words(line) end }
end


def write_out
    fout_show_signs
    $fout.close
end


# Building the frequency table
=begin
start hash from $simples.uniq.sort (390 entries)
run through $simples, finding each instance and incrementing the value in the hash
print hash
=end

def freq(ary)  # ary = eg $simples
    $signsuniq = ary.sort.uniq  # holding the unique uruk iv signs
    
    # create frequency tables
    $signsuniq.each { |sign| $signfreq[sign] = $simples.count(sign) }  # create hash holding the frequency table
    $signfreq.sort_by {|k,v| v}.reverse.each {|ary| $signpop[ary[0]] = ary[1]}   # create freq table in sorted order descending by value and convert back to hash
end

def show_freq(h)  # h = eg $signpop
    h.each {|key,value| print key; print(": "); puts value }  # print freq table in alphabet order by word
end

def write_freq(h)  # h = eg $signpop
    h.each {|key,value| $freqout.print key; $freqout.print(": "); $freqout.puts value }  # print freq table in alphabet order by word
    $freqout.fclose
end


# Application
def runApp
    setupApp
    clear_signs
    show_signs
    all_words
    show_signs
    write_out
    freq $simples
    show_freq $signpop
    write_freq $signpop
end

####
puts "Building CDLI sign frequency table for Uruk IV period tablets"
runApp


# test
# $simples.count($signsuniq[0])
# $h[$signsuniq[0]] = $simples.count($signsuniq[0])




=begin
# ?? what are [sign] ?

# safe symbols to split on are # @ and ? (after a fix to @)

$a=[]
$b=[]
# safe to split on # and discard rest
$simples.grep(/#/).each { |word| x=word.split(/#/); $a.push(x[0]); $b.push(x[1]); }
puts $b.join("   ")

$a=[]
$b=[]
# make @ fix (x1) then safe to split on @ and discard rest
$simples.grep(/@/).each { |word| x=word.split(/@/); $a.push(x[0]); $b.push(x[1]); }
puts $b.join("   ")

$a=[]
$b=[]
# safe to split on ? and discard rest
$simples.grep(/\?/).each { |word| x=word.split(/\?/); $a.push(x[0]); $b.push(x[1]); }
puts $b.join("   ")



# separate simple and compound lists

$simple=$sign4[0..$sign4.rindex("ZI~b")]
$simple.uniq!

# for simple words it is safe to merge tilde variants
$a=[]
$b=[]
$simples.grep(/~/).each { |word| x=word.split(/~/); $a.push(x[0]); $b.push(x[1]); }
puts $b.join("   ")

$simple2=[]
$simple.each {|word| $simple2.push(word.split(/~/)[0])}
$simple2.uniq!

# compound lists have words joined with & . or x, starts and ends with | 
$compound=$sign4[($sign4.rindex("ZI~b")+1)..$sign4.length]
$compound.uniq!

# compound words are made from x . & 
# NOT safe to split compound words
$a=[]
$b=[]
$signlist.grep(/&/).each { |word| x=word.split(/x/); $a.push(x[0]); $b.push(x[1]); }

# ?? do any compound words have simple parts that are new?

### cleaning up main list in order to do the frequency count.

=end